'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.endCommandBuffer = exports.endCommand = exports.NextionProtocol = exports.nextionProtocol = undefined;

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.read = read;

var _binProtocol = require('bin-protocol');

var _codes = require('./codes');

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NextionProtocol = (0, _binProtocol.createProtocol)(function () {
  var reset = this.reader.reset;
  this.reader.reset = function (buf) {
    if (buf.slice(buf.length - 3).equals(endCommandBuffer)) {
      return reset.call(this, buf.slice(0, -3));
    }
    return reset.call(this, buf);
  };
});

var readers = {
  byte(name) {
    this.UInt8(name);
  },
  code() {
    this.byte('code');
    if (!_codes.codes[this.context.code]) {
      throw new Error(`Unknown code: ${this.context.code}`);
    }
    return _codes.codes[this.context.code];
  },
  touchEvent() {
    this.pageId().byte('buttonId').byte('releaseEvent');
    this.context.releaseEvent = Boolean(this.context.releaseEvent);
  },
  pageId() {
    this.byte('pageId');
  },
  touchCoordinate() {
    this.byte('xHigh').byte('xLow').byte('yHigh').byte('yLow').touchEvent();
  },
  wake() {
    this.touchCoordinate();
  },
  stringData() {
    this.context.value = this.buffer.toString();
  },
  numericData() {
    this.Int16LE('value');
  }
};

(0, _keys2.default)(readers).forEach(function (name) {
  NextionProtocol.define(name, {
    read: readers[name]
  });
});

function read(data) {
  if (!Buffer.isBuffer(data)) {
    data = Buffer.from(data);
  }
  var codeByte = data.slice(0, 1);
  var code = nextionProtocol.read(codeByte).code().result;
  var result = {
    code
  };
  var reader = nextionProtocol.read(data.slice(1));
  if (_fp2.default.isFunction(reader[code])) {
    result.data = reader[code]().result;
  }
  return result;
}

var nextionProtocol = exports.nextionProtocol = new NextionProtocol();

exports.NextionProtocol = NextionProtocol;
var endCommand = exports.endCommand = [0xff, 0xff, 0xff];
var endCommandBuffer = exports.endCommandBuffer = Buffer.from(endCommand);
//# sourceMappingURL=protocol.js.map