'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UART = exports.DEFAULT_PARSER = exports.DEFAULT_BAUD_RATE = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _events = require('events');

var _serialport = require('serialport');

var _serialport2 = _interopRequireDefault(_serialport);

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _protocol = require('./protocol');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_BAUD_RATE = exports.DEFAULT_BAUD_RATE = 9600;
var DEFAULT_PARSER = exports.DEFAULT_PARSER = _serialport2.default.parsers.byteDelimiter(_protocol.endCommand);

var PORT_GUESS_REGEX = /usb|acm|^com/i;

var txDebug = (0, _debug2.default)('nextion:uart:tx');
var rxDebug = (0, _debug2.default)('nextion:uart:rx');

var isValidPort = _fp2.default.allPass([_fp2.default.isObject, _fp2.default.pipe(_fp2.default.property('on'), _fp2.default.isFunction), _fp2.default.pipe(_fp2.default.property('write'), _fp2.default.isFunction)]);

/**
 * Whether or not to expect the Nextion device to return success/failure codes
 * after each serial command.  Defaults to "always".
 * @typedef {string|boolean|number} ReturnCommandResultValue
 */

/**
 * Options for creating a UART option.  Some properties may be ignored if a
 * Serialport or similar object has been provided when calling the factory
 * functions.
 * @typedef {Object} UARTOptions
 * @property {ReturnCommandResultValue} returnCommandResult - Value
 * @property {string} port - Name (e.g. "COM3") or path (e.g. "/dev/ttyUSB0")
 * @property {number} baudRate - Defaults to 9600
 * @property {Function} parser - Serialport parser; defaults to byte delimiter
 */

/**
 * Wraps a SerialPort or similar object; provides convenience methods for
 * interaction with a Nextion over UART.
 */

var UART = exports.UART = function (_EventEmitter) {
  (0, _inherits3.default)(UART, _EventEmitter);

  /**
   * Sets some default options
   * @param {events.EventEmitter|stream.Duplex} port - Serial port interface
   * @param {UARTOptions} [opts={}] - Options
   */
  function UART(port) {
    var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck3.default)(this, UART);

    var _this = (0, _possibleConstructorReturn3.default)(this, (UART.__proto__ || (0, _getPrototypeOf2.default)(UART)).call(this));

    if (!isValidPort(port)) {
      throw new TypeError('"port" must be a Serialport-like object');
    }

    _this.port = port;

    _this.opts = _fp2.default.defaults({
      returnCommandResult: 'always'
    }, opts);
    return _this;
  }

  (0, _createClass3.default)(UART, [{
    key: 'assertPortOpen',


    /**
     * Asserts {@link UART#port} is open.
     * @throws {Error} Port object must be truthy and not closed.
     */
    value: function assertPortOpen() {
      if (!this.port || this.port.isClosed === true) {
        throw new Error('port not open!');
      }
    }

    /**
     * Set variable `variableName` to value `value`.
     * @param {string} variableName - Name of variable, component, system var,
     *   page, etc.
     * @param {*} value - Likely a primitive.
     * @returns {Promise<*>}
     */

  }, {
    key: 'setValue',
    value: function () {
      var _ref = (0, _bluebird.method)(function (variableName, value) {
        return this.request(`${variableName}=${value}`);
      });

      function setValue(_x2, _x3) {
        return _ref.apply(this, arguments);
      }

      return setValue;
    }()

    /**
     * Sends a raw command; does not wait for response.  Unless you really can't
     * guarantee a response, don't use this method, and use {@link UART#request}
     * instead.
     * @param {string} command - Raw ASCII command
     * @returns {UART}
     * @fires UART#error
     */

  }, {
    key: 'send',
    value: function send(command) {
      try {
        this.assertPortOpen();
        this.port.write(command);
        this.port.write(_protocol.endCommandBuffer);
        txDebug(`Sent: ${command}`);
      } catch (err) {
        /**
         * Emitted if {@link UART#assertPortOpen} throws.
         * @type Error
         * @event UART#error
         */
        this.emit('error', err);
      }
      return this;
    }

    /**
     * Makes a request to a Nextion device, expecting a response.
     * @param {Array<string>|string} commands - Command(s) to execute
     * @returns {Promise<*>} Result or array of results
     */

  }, {
    key: 'request',
    value: function () {
      var _ref2 = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee() {
        var _this2 = this;

        var commands = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
        var results;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.prev = 0;

                this.assertPortOpen();
                _context.next = 8;
                break;

              case 4:
                _context.prev = 4;
                _context.t0 = _context['catch'](0);

                this.emit('error', _context.t0);
                return _context.abrupt('return');

              case 8:
                commands = [].concat(commands);
                _context.next = 11;
                return _bluebird2.default.map(commands, function (command) {
                  _this2.send(command);
                  return new _bluebird2.default(function (resolve) {
                    _this2.port.once('data', function (data) {
                      var result = (0, _protocol.read)(data);
                      rxDebug(`Received: %j`, result);
                      resolve(result);
                    });
                  });
                }, { concurrency: 1 });

              case 11:
                results = _context.sent;
                return _context.abrupt('return', commands.length === 1 ? results.shift() : results);

              case 13:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[0, 4]]);
      }));

      function request(_x4) {
        return _ref2.apply(this, arguments);
      }

      return request;
    }()

    /**
     * Begins listening for data on the open serial port.
     * If an event is emitted from the Nextion device, an event with a
     * human-readable name will be emitted, along with any extra data, if
     * present.
     * Errors bubble up from the serial port object thru the UART instance.
     * @param {events.EventEmitter|stream.Duplex} [port] - `Serialport`-like
     *   object to listen for data and errors on.  Defaults to {@link UART#port}.
     * @returns {UART}
     */

  }, {
    key: 'bind',
    value: function bind(port) {
      var _this3 = this;

      port = port || this.port;

      port.on('data', function (data) {
        rxDebug(`'Data: ${data.toString()}`);
        try {
          var result = (0, _protocol.read)(data);
          _this3.emit('event', result.code, result.data);
          _this3.emit(result.code, result.data);
        } catch (err) {
          _this3.emit('error', err);
        }
      });

      port.on('error', function (err) {
        _this3.emit('error', err);
      });

      return this;
    }

    /**
     * Tell the Nextion device to wait (default) or not wait for responses when
     * requests are made.
     * @param {string|number|boolean} [value='always'] - false, 0 or "none" to
     *   disable waiting for responses; true, 1 or "always" to enable waiting for
     *   responses.
     * @returns {UART}
     */

  }, {
    key: 'setReturnCommandResult',
    value: function setReturnCommandResult() {
      var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'always';

      var newValue = void 0;
      switch (value) {
        case false:
        case 0:
        // falls through
        case 'none':
          newValue = 0;
          break;
        case 3:
        case true:
        // falls through
        case 'always':
        default:
          newValue = 3;
      }
      this.send(`bkcmd=${newValue}`);
      this.opts.returnCommandResult = value;
    }

    /**
     * Stops listening on the serial port, closes it, destroys the reference,
     * etc.
     * @returns {UART}
     */

  }, {
    key: 'unbind',
    value: function unbind() {
      this.port.removeAllListeners('data');
      this.port.removeAllListeners('error');
      if (_fp2.default.isFunction(this.port.close)) {
        this.port.close();
      }
      this.port = null;
      delete this.port;
      return this;
    }

    /**
     * Given a {@link Serialport}-like object, create a {@link UART} wrapper.
     * Returns a `Promise` for consistency.
     * @param {events.EventEmitter|stream.Duplex} serialPort - Serial port
     *   interface
     * @param {UARTOptions} [opts={}] - Options
     * @returns {Promise.<UART>}
     */

  }, {
    key: 'returnCommandResult',
    get: function get() {
      return this.opts.returnCommandResult;
    },
    set: function set(value) {
      this.setReturnCommandResult(value);
    }
  }], [{
    key: 'fromSerial',
    value: function () {
      var _ref3 = (0, _bluebird.method)(function (serialPort) {
        var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        return new UART(serialPort, opts);
      });

      function fromSerial(_x7, _x8) {
        return _ref3.apply(this, arguments);
      }

      return fromSerial;
    }()

    /**
     * Given a serial port name or path, or object containing one, create a
     * `Serialport` instance, open the serial port, then return a `UART`
     * instance.
     * If no port name is present, we'll try to detect the port.
     * @param {string|UARTOptions} [portName] - Serial port name or path, or
     *   options
     * @param {UARTOptions} [opts={}] - Options
     * @returns {Promise.<UART>}
     */

  }, {
    key: 'fromPort',
    value: function () {
      var _ref4 = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee2(portName) {
        var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var serialPort;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (_fp2.default.isObject(portName)) {
                  opts = portName;
                  portName = portName.port;
                }

                opts = _fp2.default.defaults({
                  parser: DEFAULT_PARSER,
                  baudRate: DEFAULT_BAUD_RATE
                });

                if (portName) {
                  _context2.next = 8;
                  break;
                }

                _context2.next = 5;
                return UART.findPort();

              case 5:
                portName = _context2.sent;

                if (portName) {
                  _context2.next = 8;
                  break;
                }

                throw new Error('Could not find a serial device!');

              case 8:
                _context2.next = 10;
                return _bluebird2.default.fromNode(function (done) {
                  // eslint-disable-next-line no-new
                  new _serialport2.default(portName, opts, done);
                });

              case 10:
                serialPort = _context2.sent;
                return _context2.abrupt('return', UART.fromSerial(serialPort, opts));

              case 12:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function fromPort(_x10, _x11) {
        return _ref4.apply(this, arguments);
      }

      return fromPort;
    }()

    /**
     * Convenience wrapper of {@link UART.fromPort} and {@link UART.fromSerial}.
     * @param {string|events.EventEmitter|stream.Duplex|UARTOptions} [port] - Port
     * @param {UARTOptions} [opts] - Options
     * @returns {Promise.<UART>}
     */

  }, {
    key: 'from',
    value: function () {
      var _ref5 = (0, _bluebird.method)(function (port) {
        var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        return _fp2.default.isObject(port) && _fp2.default.isFunction(port.on) ? UART.fromSerial(port, opts) : UART.fromPort(port, opts);
      });

      function from(_x13, _x14) {
        return _ref5.apply(this, arguments);
      }

      return from;
    }()

    /**
     * Tries to find a device on a serial/COM port.
     * @returns {Promise<string|void>} Name/path of promising serial port, if any
     */

  }, {
    key: 'findPort',
    value: function () {
      var _ref6 = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee3() {
        var allPorts;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return _bluebird2.default.fromNode(_serialport2.default.list);

              case 2:
                allPorts = _context3.sent;
                return _context3.abrupt('return', _fp2.default.pipe(_fp2.default.pluck('comName'), _fp2.default.filter(function (portName) {
                  return PORT_GUESS_REGEX.test(portName);
                }), _fp2.default.head)(allPorts));

              case 4:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function findPort() {
        return _ref6.apply(this, arguments);
      }

      return findPort;
    }()
  }]);
  return UART;
}(_events.EventEmitter);
//# sourceMappingURL=uart.js.map